struct Sprite{
	//Parametros inicializan al usar las funciones de carga
	int idRAM;						//0-255
	int idVRAM;						//0-127
	int id;							//0-127
	int idPaletaRAM;				//0-63
	int idPaletaVRAM;				//0-15
	
	//Parametros que definen el Sprite
	int ancho, alto;				//Ancho y alto minimo de la imagen segun la especificacon de la DS
	int anchoReal, altoReal;		//Ancho y alto real del sprite usado para las colisiones
	Pantalla pantalla;				//Id de la pantalla en la que se mostrara (0 arriba, 1 abajo)
	int posXPantalla, posYPantalla;	//Posicion en la que se dibujara el Sprite en la pantalla
	int posXMapa, posYMapa;			//Posicion que tendria respecto al mapa completo
									//(para cargar los sprite cuando se acerquen a la pantalla)
	int velocidad;					//Velocidad a la que se mueve el Sprite
	int velAnimacion;				//Tiempo que tarda en cambiar la animacion
	int velActual;					//Contador para saber en que momento de la animacion estamos
	int paso;						//Posicion de movimiento en la que se encuentra
	Direccion mira;					//Direccion a la que mira el Sprite
	char ruta[128];					//Ruta de la imagen
	char paleta[128];				//Ruta de la paleta
	bool tieneAsociado;				//Indica si hay un Sprite que depende de nosotros, el Sprite que depende de 
									//nosotros esta dibujado siempre debajo nuesra
	Sprite *asociado;				//Puntero a Sprite que depende de este
	bool mostrado;					//Dice si esta oculto el Sprite, solo util si esta creado

	Fila movimiento;

	char mensaje[128];
	u16 tamMensaje;

	//Funciones

	//Inicializa el Sprite
	Sprite();
	//Carga a la posicion indicada de RAM el Sprite si no tiene dependencia
	void cargarRAM(int i);
	//Carga en las posiciones de RAM indicadas el Sprite y su dependiente.
	void cargarRAM(int i, int j);
	//Carga a la posicion indicada de RAM la paleta del Sprite
	void cargaPaletaRAM(int i);	
	//Borra el Sprite de la RAM
	void descargarRAM();
	//Borra la paleta del Sprite de la RAM
	void descargarPaletaRAM();
	//Carga Sprite de RAM a la posicion indicada de VRAM (ahorro a True indica que solo se copia un Frame)
	//Solo si no tiene dependencias
	void cargarVRAM(int i, bool ahorro);
	//Carga Sprite de RAM a la posicion indicada de VRAM (ahorro a True indica que solo se copia un Frame)
	//Solo si tiene dependencias
	void cargarVRAM(int i,int j, bool ahorro);
	//Carga la paleta de RAM a la posicion indicada de VRAM
	void cargarPaletaVRAM(int i);
	//Borra el Sprite de la VRAM
	void descargaVRAM();
	//Crea y muestra el Sprite en la posicion de pantalla y direccion indicada asignandole el identificador
	//que pasamos como parametro, solo para el caso de Sprite que no tengan dependencias
	//Si pantalla es "true" indica que le estamos pasando las coordenadas respecto a la pantalla, en caso
	//contrario respecto al Mapa
	void crear(int i, int X, int Y, bool posEnPantalla, Direccion d);
	//Crea y muestra el Sprite "compuesto" en la posicion de pantalla y direccion indicada asignandole a
	//cada uno el identificador que pasamos como parametro
	//Si pantalla es "true" indica que le estamos pasando las coordenadas respecto a la pantalla, en caso
	//contrario respecto al Mapa
	void crear(int i, int j, int X, int Y, bool posEnPantalla, Direccion d);
	//Borra el Sprite, los datos se mantienen en la VRAM
	void borrar();
	//Indica en que capa debe dibujarse el Sprite (si nadie depende de el)
	void setCapa(Capa c);
	//Indica en que capa debe dibujarse cada una de las dos partes del Sprite (debe tener dependencia)
	void setCapa(Capa c1, Capa c2);
	//Pone visible el Sprite si previamente a sido ocultado.
	void mostrar();
	//Pone invisible el Sprite, aunque este sigue existiendo.
	void ocultar();
	//Nos indica si grafico esta oculto
	bool estaOculto();
	//Voltea el grafico horizontalmente si b = "true" en caso contrario lo devuelve a su forma orifinal
	//si ha sido previamente volteado
	void voltearHorizontal(bool b);
	//Indica si el grafico esta volteado horizontalmente
	bool getVolteadoHorizontal();
	//Voltea el grafico verticalmente si b = "true" en caso contrario lo devuelve a su forma orifinal
	//si ha sido previamente volteado
	void voltearVertical(bool b);
	//Indica si el grafico esta volteado verticalmente
	bool getVolteadoVertical();
	//Muestra del sprite el Frame (porcion) indicado
	void frame(int f);
	//Pone el Sprite en la direccion indicada (tanto si es o no compuesto), si comprobar es "true" comprueba si es compuesto
	//en caso contrario actua con el como si no lo fuera
	void mirar(Direccion d, bool comprobar);

	//Funciones de verificacion
	//Funcion que devuelve:
		//-1 Si la paleta no esta asignada
		//indice del array paletas[64] que indica donde tenemos almacenada la informacion de carga de esa paleta
	int paletaRegistrada();
	//registra la paleta en el array, si RAM = "true" modifica su valor de ram, en caso contrario el de vram
	void registraPaleta(int i, bool ram);
	//Nos devuelve si el Sprite esta en el radio de "dibujo" para ser creado
	bool estaCerca();
	//Nos indica si el Sprite se deberia ver en la pantalla segun su posivion en el Mapa
	bool enPantalla();
	//Mueve el Sprite a la posicion dada de la pantalla si "posEnPantalla" = "true", del mapa en caso contrario
	void mover(int x, int y, bool posEnPantalla);
	//Mueve el Sprite en la direccion indicada a la velocidad del mismo.
	void mover(Direccion d);
	//"Arrastra" el Sprite en la direccion contraria a la indicada tantas pixeles como los pasados
	void arrastrar(Direccion d, int pixeles);
	//Anima al personaje simulando movimiento en la direccion indicada
	void animar(Direccion d);
	//Comprueba si el Sprite se puede mover en una direccion determinada segun el mapa de colision que se le pasa
	bool puedeMover(Direccion d, u8 mapa);
	//Actualiza la posicion de las coordenadas en el mapa (no en pantalla), los parametros es la posicion en el mapa
	//de la esquina superior izquierda
	void actualizaPosicionMapa(int x, int y);
	//Cambia el texto que "dice" el Sprite por el que se le pasa como parametro
	void cambiaTexto(char *texto);
};

Sprite::Sprite(){
	tieneAsociado = false;
	mostrado = false;
}

void Sprite::cargarRAM(int i){
	if (!tieneAsociado){
		idRAM = i;
		NF_LoadSpriteGfx(ruta, idRAM, ancho, alto);
	}
}

void Sprite::cargarRAM(int i, int j){
	idRAM = i;
	NF_LoadSpriteGfx(ruta, idRAM, ancho, alto);
	asociado->cargarRAM(j);
}

void Sprite::cargaPaletaRAM(int i){
	int s = paletaRegistrada();
	if (s == -1){
		registraPaleta(i,true);
		idPaletaRAM = i;
		NF_LoadSpritePal(paleta, idPaletaRAM);
	}else
		idPaletaRAM = paletas[s].ram;

	if (tieneAsociado){
		//ESTO PUEDE QUE HAYA QUE CAMBIARLO A UNA LLAMADA A ESTA MISMA FUNCION
		asociado->idPaletaRAM = idPaletaRAM;
	}
}

void Sprite::descargarRAM(){
	NF_UnloadSpriteGfx(idRAM);
}

void Sprite::descargarPaletaRAM(){
	NF_UnloadSpritePal(idPaletaRAM);
}

void Sprite::cargarVRAM(int i, bool ahorro){
	if (!tieneAsociado){
		
		idVRAM = i;
		NF_VramSpriteGfx(pantalla, idRAM, idVRAM, ahorro);
	}
}

void Sprite::cargarVRAM(int i,int j, bool ahorro){
	idVRAM = i;
	NF_VramSpriteGfx(pantalla, idRAM, idVRAM, ahorro);
	asociado->cargarVRAM(j, ahorro);
}

void Sprite::cargarPaletaVRAM(int i){
	
	int s = paletaRegistrada();
	if (s == -1){
		registraPaleta(i,false);
		idPaletaVRAM = i;
		NF_VramSpritePal(pantalla, idPaletaRAM, idPaletaVRAM);
	}else if (paletas[s].vram == -1){
		registraPaleta(i,false);
		idPaletaVRAM = i;
		NF_VramSpritePal(pantalla, idPaletaRAM, idPaletaVRAM);
	}else
		idPaletaVRAM = paletas[s].vram;

	if (tieneAsociado){
		//ESTO PUEDE QUE HAYA QUE CAMBIARLO A UNA LLAMADA A ESTA MISMA FUNCION
		asociado->idPaletaVRAM = idPaletaVRAM;
	}
}

void Sprite::descargaVRAM(){
	NF_FreeSpriteGfx(pantalla, idVRAM);
}

void Sprite::crear(int i, int X, int Y, bool posEnPantalla, Direccion d){
	id = i;
	bool creado = false;

	if (posEnPantalla){
		posXPantalla = X;
		posYPantalla = Y;
		posXMapa = MapaX + X;
		posYMapa = MapaY + Y;
		if (enPantalla()){
			NF_CreateSprite(pantalla, id, idVRAM, idPaletaVRAM, posXPantalla, posYPantalla);
			mostrado = true;
			creado = true;
		}else if (estaCerca()){
			NF_CreateSprite(pantalla, id, idVRAM, idPaletaVRAM, posXPantalla, posYPantalla);
			creado = true;
			ocultar();
		}
	}
	else{
		posXMapa = X;
		posYMapa = Y;
		posXPantalla = posXMapa - MapaX;
		posYPantalla = posYMapa - MapaY;
		if (enPantalla()){
			NF_CreateSprite(pantalla, id, idVRAM, idPaletaVRAM, posXPantalla, posYPantalla);
			mostrado = true;
			creado = true;
		}else if (estaCerca()){
			NF_CreateSprite(pantalla, id, idVRAM, idPaletaVRAM, posXPantalla, posYPantalla);
			creado = true;
			ocultar();
		}
	}
	if (creado && !tieneAsociado)
		mirar(d, true);
}

void Sprite::crear(int i,int j, int X, int Y, bool posEnPantalla, Direccion d){
	id = i;
	bool creado = false;

	if (posEnPantalla){
		posXPantalla = X;
		posYPantalla = Y;
		posXMapa = MapaX + X;
		posYMapa = MapaY + Y;
		if (enPantalla()){
			NF_CreateSprite(pantalla, id, idVRAM, idPaletaVRAM, posXPantalla, posYPantalla);
			asociado->crear(j,X,Y+alto,pantalla,d);
			mostrado = true;
			creado = true;
		}else if (estaCerca()){
			NF_CreateSprite(pantalla, id, idVRAM, idPaletaVRAM, posXPantalla, posYPantalla);
			asociado->crear(j,X,Y+alto,pantalla,d);
			creado = true;
			ocultar();
		}
	}
	else{
		posXMapa = X;
		posYMapa = Y;
		posXPantalla = posXMapa - MapaX;
		posYPantalla = posYMapa - MapaY;
		if (enPantalla()){
			NF_CreateSprite(pantalla, id, idVRAM, idPaletaVRAM, posXPantalla, posYPantalla);
			asociado->crear(j,X,Y+alto,pantalla,d);
			mostrado = true;
			creado = true;
		}else if (estaCerca()){
			NF_CreateSprite(pantalla, id, idVRAM, idPaletaVRAM, posXPantalla, posYPantalla);
			asociado->crear(j,X,Y+alto,pantalla,d);
			creado = true;
			ocultar();
		}
	}
	
	if (creado)
		mirar(d, false);
}


void Sprite::borrar(){
	NF_DeleteSprite(pantalla, id);
	if (tieneAsociado)
		asociado->borrar();
}

void Sprite::setCapa(Capa c){
	NF_SpriteLayer(pantalla, id, c);
}

void Sprite::setCapa(Capa c1, Capa c2){
	NF_SpriteLayer(pantalla, id, c1);
	if (tieneAsociado)
		asociado->setCapa(c2);
}

void Sprite::mostrar(){
	NF_ShowSprite(pantalla, id, true);
	mostrado = true;
}
void Sprite::ocultar(){
	NF_ShowSprite(pantalla, id, false);
	mostrado = false;
}

bool Sprite::estaOculto(){
	return mostrado;
}

void Sprite::voltearHorizontal(bool b){
	if (b != getVolteadoHorizontal())
		NF_HflipSprite(pantalla, id, b);
}

bool Sprite::getVolteadoHorizontal(){
	return NF_GetSpriteHflip(pantalla, id);
}

void Sprite::voltearVertical(bool b){
	if (b != getVolteadoVertical())
		NF_VflipSprite(pantalla, id, b);
}

bool Sprite::getVolteadoVertical(){
	return NF_GetSpriteVflip(pantalla, id);
}

void Sprite::frame(int f){
	NF_SpriteFrame(pantalla,id,f);
}

void Sprite::mirar(Direccion d, bool comprobar){
	int base;
	base = d;
	if (d == izquierda){
		base = derecha;
		voltearHorizontal(true);
	}else{
		base = d;
		voltearHorizontal(false);
	}
	mira = d;
	velActual = 0;
	paso = 0;
	frame(base);

	if (comprobar && tieneAsociado)
		asociado->mirar(d, false);
}

int Sprite::paletaRegistrada(){
	int s = -1;
	for (int i=0;(i<64)&&(s==-1);i++){
		if (paletas[i].usada){
			if (strcmp(paletas[i].nombre,paleta) == 0)
				s = i;
		}
	}
	return s;
}

void Sprite::registraPaleta(int i, bool ram){
	if (paletaRegistrada() == -1){
		for (int j=0;(j<64);j++){
			if (!paletas[j].usada){
				paletas[j].usada = true;
				if (ram)
					paletas[j].ram = i;
				else
					paletas[j].vram = i;
				strcpy(paletas[j].nombre,this->paleta);
			}
		}
	}
}

bool Sprite::estaCerca(){
	if (!asociado)
		return ((MapaX - radioCarga) < (posXMapa + ancho)) && ((MapaX + radioCarga + 255) > posXMapa) && ((MapaY - radioCarga) < (posYMapa + alto)) && ((MapaY + radioCarga + 191) > posYMapa);
	else
		return ((MapaX - radioCarga) < (posXMapa + ancho)) && ((MapaX + radioCarga + 255) > posXMapa) && ((MapaY - radioCarga) < (posYMapa + alto + asociado->alto)) && ((MapaY + radioCarga + 191) > posYMapa);
}

bool Sprite::enPantalla(){
	return (MapaX <= posXMapa + ancho) && ((MapaX + 255) >= posXMapa) && (MapaY <= posYMapa + alto) && ((MapaY + 191) >= posYMapa);
}

void Sprite::mover(int x, int y, bool posEnPantalla){
	if (posEnPantalla){
		posXPantalla = x;
		posYPantalla = y;
		posXMapa = MapaX + x;
		posYMapa = MapaY + y;
	}
	else{
		posXMapa = x;
		posYMapa = y;
		posXPantalla = posXMapa - MapaX;
		posYPantalla = posYMapa - MapaY;
	}
	if (enPantalla()){
		NF_MoveSprite(pantalla, id, posXPantalla, posYPantalla);
		if (!mostrado)
			mostrar();
	}else if (estaCerca()){
		ocultar();
	}
	if (tieneAsociado)
		asociado->mover(x, y+alto, pantalla);
}

void Sprite::mover(Direccion d){
	if (d == arriba){
		posYPantalla -= velocidad;
		posYMapa -= velocidad;
	}else if (d == abajo){
		posYPantalla += velocidad;
		posYMapa += velocidad;
	}else if (d == derecha){
		posXPantalla += velocidad;
		posXMapa += velocidad;
	}else{
		posXPantalla -= velocidad;
		posXMapa -= velocidad;
	}
	if (enPantalla()){
		NF_MoveSprite(pantalla, id, posXPantalla, posYPantalla);
		if (!mostrado)
			mostrar();
	}else if (estaCerca()){
		ocultar();
	}
	if (tieneAsociado)
		asociado->mover(d);
}

void Sprite::animar(Direccion d){
	int base;
	if (mira != d)
		mirar(d,true);
	else{
		velActual++;
		velActual = velActual % velAnimacion;
		if (velActual == (velAnimacion - 1)){
			paso++;
			paso = paso % 4;	//Dividimos por 4 porque da un valor entero entre 0 y 3, uno para cada posicion de la animacion
			base = d;
			if (d == izquierda){
				base = derecha;
				voltearHorizontal(true);
			}else
				voltearHorizontal(false);
			frame(base + paso);
			if (tieneAsociado)
				asociado->frame(base + paso);
		}
	}
}

bool Sprite::puedeMover(Direccion d, u8 mapa){
	int x,y;
	int x2,y2;
	Sprite *s;
	if (!tieneAsociado)
		s = this;
	else
		s = this->asociado;

	x = s->posXMapa;
	y = s->posYMapa;
	x2 = s->posXMapa+8;
	y2 = s->posYMapa+8;
	switch (d){
		case arriba:
			x += 8;
			//y = s->posYMapa;

			y2 -=32;
			break;
		case abajo:
			x += 8;
			y += 24;

			y2 +=32;
			break;
		case derecha:
			x += 24;
			y += 8;

			x2 +=32;
			break;
		case izquierda:
			//x = s->posXMapa;
			y += 8;
			
			x2 -=32;
			break;
		default:
			break;
	}
	y = y - defTiles;// + 8;
	y2 = y2 - defTiles;// + 8;
	return (NF_GetTile(mapa,x,y) == tSuelo) && (NF_GetTile(mapa,x2,y2) != tNPC);
}


void Sprite::actualizaPosicionMapa(int x, int y){
	this->posXMapa = x + this->posXPantalla;
	this->posYMapa = y + this->posYPantalla;
	if (tieneAsociado)
		asociado->actualizaPosicionMapa(x,y);
}

void Sprite::arrastrar(Direccion d,int pixeles){
	if (d == arriba){
		posYPantalla += pixeles;
		//posYMapa += pixeles;
	}else if (d == abajo){
		posYPantalla -= pixeles;
		//posYMapa -= pixeles;
	}else if (d == derecha){
		posXPantalla -= pixeles;
		//posXMapa -= pixeles;
	}else{
		posXPantalla += pixeles;
		//posXMapa += pixeles;
	}
	if (enPantalla()){
		NF_MoveSprite(pantalla, id, posXPantalla, posYPantalla);
		if (!mostrado)
			mostrar();
	}else if (estaCerca()){
		ocultar();
	}
	if (tieneAsociado)
		asociado->arrastrar(d,pixeles);
}

void Sprite::cambiaTexto(char *texto){
	tamMensaje = strlen(texto);
	NF_DmaMemCopy(mensaje,texto,strlen(texto)+1);

}
//FALTA POR HACER
/*
	
*/